import 'package:fluro/fluro.dart';
import 'package:flutter/material.dart';
import 'package:chart/models/app_model.dart';
import 'package:provider/provider.dart';
import 'package:chart/router/routers.dart';
import 'package:chart/router/application.dart';
import 'package:flutter/services.dart' as services;
import 'package:chart/constant/app_colors.dart';
import 'package:chart/constant/app_strings.dart';
import 'package:flutter_localizations/flutter_localizations.dart'; //国际化

void main() async {
  WidgetsFlutterBinding.ensureInitialized();
  var appModel = AppModel();
  // var userViewModel = UserModel();
  // var userSubscribeModel = UserSubscribeModel();
  // var serverModel = ServerModel();
  // var planModel = PlanModel();
  runApp(MultiProvider(providers: [
    ChangeNotifierProvider<AppModel>.value(value: appModel),
    // ChangeNotifierProvider<UserModel>.value(value: userViewModel),
    // ChangeNotifierProvider<UserSubscribeModel>.value(value: userSubscribeModel),
    // ChangeNotifierProvider<ServerModel>.value(value: serverModel),
    // ChangeNotifierProvider<PlanModel>.value(value: planModel)
  ], child: ChartApp()));
}

class ChartApp extends StatelessWidget {
  ChartApp({Key? key}) : super(key: key) {
    final router = FluroRouter();
    Routers.configureRoutes(router);
    Application.router = router;
  }

  // This widget is the root of your application.
  @override
  Widget build(BuildContext context) {
    services.SystemChrome.setPreferredOrientations([
      services.DeviceOrientation.portraitUp,
      services.DeviceOrientation.portraitDown
    ]);

    return MaterialApp(
      // <--- /!\ Add the builder
      title: AppStrings.appName,
      navigatorKey: Application.navigatorKey,
      debugShowCheckedModeBanner: false,
      onGenerateRoute: Application.router?.generator,
      localizationsDelegates: const [
        // 本地化的代理类
        GlobalMaterialLocalizations.delegate,
        GlobalWidgetsLocalizations.delegate,
      ],
      supportedLocales: const [
        Locale('en', 'US'), // 美国英语
        Locale('zh', 'CN'), // 中文简体
        //其它Locales
      ],
      theme: ThemeData(
          primarySwatch: AppColors.themeColor,
          visualDensity: VisualDensity.adaptivePlatformDensity),
    );
  }
}
