import 'dart:convert';
import 'dart:io';
import 'package:flutter/material.dart';
import 'package:flutter_chat_types/flutter_chat_types.dart' as types;
import 'package:flutter_chat_types/flutter_chat_types.dart';
import 'package:flutter_chat_ui/flutter_chat_ui.dart';
import 'package:flutter_screenutil/flutter_screenutil.dart';
import 'package:provider/provider.dart';
import 'package:chart/constant/app_strings.dart';
import 'package:chart/constant/app_colors.dart';
import 'package:chart/constant/app_dimens.dart';
import 'package:intl/date_symbol_data_local.dart';
import 'package:file_picker/file_picker.dart';
import 'package:image_picker/image_picker.dart';
import 'package:chart/models/app_model.dart';
import 'package:chart/models/user_model.dart';
import 'package:chart/widgets/sail_app_bar.dart';
import 'package:uuid/uuid.dart';
import 'package:mime/mime.dart';
import 'package:flutter/services.dart';
import 'package:chart/service/message_service.dart';
import 'package:chart/pages/webview_widget.dart';
import 'package:bubble/bubble.dart';
import 'package:animated_text_kit/animated_text_kit.dart';
import 'package:fluttertoast/fluttertoast.dart';

final _user = const types.User(
  id: '82091008-a484-4a89-ae75-a22bf8d6f3ac',
);

final _user1 = const types.User(
  id: '82091008-a4aa-4a89-ae75-a22bf8d6f3aa',
);

final receiveUser = const types.User(
    id: '82091008-a484-4a89-ae75-a22bf8d6f3aa',
    firstName: "GPT",
    lastName: '大师',
    imageUrl:
        "https://imgb15.photophoto.cn/20201124/jiqirentupian-39852917_3.jpg"
    // imageUrl: "assets/images/300.jpg",
    );

class HomePage extends StatefulWidget {
  const HomePage({Key? key}) : super(key: key);

  @override
  HomePageState createState() => HomePageState();
}

class HomePageState extends State<HomePage> {
  final PageController _pageController = PageController(initialPage: 0);
  late AppModel _appModel;
  // late ServerModel _serverModel;
  late UserModel _userModel;
  // late UserSubscribeModel _userSubscribeModel;
  bool _isLoadingData = false;
  String _appTitle = '哈哈GPT';
  // int _loadingIndex = 0;
  // ignore: prefer_final_fields
  List<types.Message> _messages = [];

  final Map _tabMap = {
    0: AppStrings.appName,
    1: "login",
    2: '节点',
    3: '我的',
  };

  @override
  void initState() {
    super.initState();
    _loadMessages();
  }

  @override
  void didChangeDependencies() async {
    super.didChangeDependencies();

    _appModel = Provider.of<AppModel>(context);
  }

  void jumpToPage(int page) {
    setState(() {
      _pageController.jumpToPage(page);
      _appTitle = _tabMap[page];
      print("_appTitle: $_appTitle");
    });
  }

  @override
  Widget build(BuildContext context) {
    // WebViewWidget(
    //   name: "title",
    //   url: "https://mp.weixin.qq.com/s/AJCPG0b72g4mB3ODM555Cw");
    return AnnotatedRegion<SystemUiOverlayStyle>(
        value: SystemUiOverlayStyle.light,
        //  _appModel.isOn
        //     ? SystemUiOverlayStyle.dark
        //     : SystemUiOverlayStyle.light,
        child: Scaffold(
          appBar: SailAppBar(
            appTitle: _appTitle,
          ),

          // extendBody: true,
          // backgroundColor:
          //     _appModel.isOn ? AppColors.yellowColor : AppColors.grayColor,
          body: Chat(
            bubbleBuilder: _bubbleBuilder,
            theme: const DefaultChatTheme(
              // #1e1c39//#1e1c39#1e1c39#1e1c39
              // rgba(30, 28, 57, 1.00)
              backgroundColor: Color.fromRGBO(30, 28, 57, 1),
              // inputBackgroundColor: Colors.red,
            ),
            messages: _messages,
            // onAttachmentPressed: _handleAttachmentPressed,
            onMessageTap: _handleMessageTap,
            onPreviewDataFetched: _handlePreviewDataFetched,
            onSendPressed: _handleSendPressed,
            showUserAvatars: true,
            showUserNames: true,
            user: _user,
            l10n: const ChatL10nEn(
                inputPlaceholder: '请输入你的问题，并寻求解答...',
                emptyChatPlaceholder: '暂无聊天信息',
                sendButtonAccessibilityLabel: '发送'),
          ),
          floatingActionButtonLocation:
              FloatingActionButtonLocation.centerDocked,
        ));
  }

  void _addMessage(types.Message message) {
    setState(() {
      _messages.insert(0, message);
    });
  }

  void _cancelLoading() {
    var data;
    // _messages.forEach((element) {
    //   print(element.props);
    // });
    data = _messages.firstWhere((element) => element.status == Status.sending);
    // _messages.where((mess)=>"${mess.}" === "loading");
    setState(() {
      _messages.remove(data);
    });
  }

  void _handleAttachmentPressed() {
    showModalBottomSheet<void>(
      context: context,
      builder: (BuildContext context) => SafeArea(
        child: SizedBox(
          height: 144,
          child: Column(
            crossAxisAlignment: CrossAxisAlignment.stretch,
            children: <Widget>[
              TextButton(
                onPressed: () {
                  Navigator.of(context).pushNamed('/login');
                },
                child: const Align(
                  alignment: AlignmentDirectional.centerStart,
                  child: Text('Photo'),
                ),
              ),
              TextButton(
                onPressed: () {
                  Navigator.pop(context);
                  _handleFileSelection();
                },
                child: const Align(
                  alignment: AlignmentDirectional.centerStart,
                  child: Text('File'),
                ),
              ),
              TextButton(
                onPressed: () => Navigator.pop(context),
                child: const Align(
                  alignment: AlignmentDirectional.centerStart,
                  child: Text('Cancel'),
                ),
              ),
            ],
          ),
        ),
      ),
    );
  }

  void _handleFileSelection() async {
    final result = await FilePicker.platform.pickFiles(
      type: FileType.any,
    );

    if (result != null && result.files.single.path != null) {
      final message = types.FileMessage(
        author: _user,
        createdAt: DateTime.now().millisecondsSinceEpoch,
        id: const Uuid().v4(),
        mimeType: lookupMimeType(result.files.single.path!),
        name: result.files.single.name,
        size: result.files.single.size,
        uri: result.files.single.path!,
      );

      _addMessage(message);
    }
  }

  void _handleImageSelection() async {
    // final result = await ImagePicker().pickImage(
    //   imageQuality: 70,
    //   maxWidth: 1440,
    //   source: ImageSource.gallery,
    // );

    // if (result != null) {
    //   final bytes = await result.readAsBytes();
    //   final image = await decodeImageFromList(bytes);

    //   final message = types.ImageMessage(
    //     author: _user,
    //     createdAt: DateTime.now().millisecondsSinceEpoch,
    //     height: image.height.toDouble(),
    //     id: const Uuid().v4(),
    //     name: result.name,
    //     size: bytes.length,
    //     uri: result.path,
    //     width: image.width.toDouble(),
    //   );

    //   _addMessage(message);
    // }
  }

  void _handleMessageTap(BuildContext _, types.Message message) async {
    // if (message is types.FileMessage) {
    //   var localPath = message.uri;

    //   if (message.uri.startsWith('http')) {
    //     try {
    //       final index =
    //           _messages.indexWhere((element) => element.id == message.id);
    //       final updatedMessage =
    //           (_messages[index] as types.FileMessage).copyWith(
    //         isLoading: true,
    //       );

    //       setState(() {
    //         _messages[index] = updatedMessage;
    //       });

    //       final client = http.Client();
    //       final request = await client.get(Uri.parse(message.uri));
    //       final bytes = request.bodyBytes;
    //       final documentsDir = (await getApplicationDocumentsDirectory()).path;
    //       localPath = '$documentsDir/${message.name}';

    //       if (!File(localPath).existsSync()) {
    //         final file = File(localPath);
    //         await file.writeAsBytes(bytes);
    //       }
    //     } finally {
    //       final index =
    //           _messages.indexWhere((element) => element.id == message.id);
    //       final updatedMessage =
    //           (_messages[index] as types.FileMessage).copyWith(
    //         isLoading: null,
    //       );

    //       setState(() {
    //         _messages[index] = updatedMessage;
    //       });
    //     }
    //   }

    //   await OpenFilex.open(localPath);
    // }
  }

  void _handlePreviewDataFetched(
    types.TextMessage message,
    types.PreviewData previewData,
  ) {
    final index = _messages.indexWhere((element) => element.id == message.id);
    final updatedMessage = (_messages[index] as types.TextMessage).copyWith(
      previewData: previewData,
    );

    setState(() {
      _messages[index] = updatedMessage;
    });
  }

  void _handleSendPressed(types.PartialText message) async {
    var data = false;
    // _messages.forEach((element) {
    //   print(element.props);
    // });

    if (_messages.isNotEmpty) {
      data = !_messages.every((element) => element.status != Status.sending);
    }

    print("data-----------${data}");

    if (data) {
      Fluttertoast.showToast(
          msg: "操作太快了 你上个问题还没问完呢",
          toastLength: Toast.LENGTH_SHORT,
          gravity: ToastGravity.CENTER,
          timeInSecForIosWeb: 1,
          backgroundColor: Colors.red,
          textColor: Colors.white,
          fontSize: 16.0);
      return;
    }

    final textMessage = types.TextMessage(
      author: _user,
      createdAt: DateTime.now().millisecondsSinceEpoch,
      id: const Uuid().v4(),
      text: message.text,
    );

    final loadingMessage = types.TextMessage(
      author: receiveUser,
      createdAt: DateTime.now().millisecondsSinceEpoch,
      id: const Uuid().v4(),
      text: "loading",
      status: Status.sending,
      // "sending",
    );

    _addMessage(textMessage);
    _addMessage(loadingMessage);

    String? result = await MessageService().getMessage(message.text);
    _cancelLoading();
    final receiveMessage = types.TextMessage(
      author: receiveUser,
      createdAt: DateTime.now().millisecondsSinceEpoch,
      id: const Uuid().v4(),
      text: "$result",
    );
    _addMessage(receiveMessage);
  }

  void _viewMessage(String value) {
    final textMessage1 = types.TextMessage(
      author: _user1,
      createdAt: DateTime.now().millisecondsSinceEpoch,
      id: const Uuid().v4(),
      text: value,
    );

    _addMessage(textMessage1);
    // // print(value);
    // print(value); //msg value: Hello, World!
  }

  void _loadMessages() {
    // final response = `""`
    // final messages = (jsonDecode(response) as List)
    //     .map((e) => types.Message.fromJson(e as Map<String, dynamic>))
    //     .toList();

    // setState(() {
    //   _messages = messages;
    // });
  }
}

Widget _bubbleBuilder(
  Widget child, {
  required message,
  required nextMessageInGroup,
}) =>
    Bubble(
      child: AnimatedTextKit(
        totalRepeatCount: 1,
        animatedTexts: identical(message.status, Status.sending)
            ? [
                WavyAnimatedText('正在思考中......'),
                WavyAnimatedText('正在使劲思考中......'),
                WavyAnimatedText('正在拼命思考中......'),
                WavyAnimatedText('您的问题太有深度，回答很困难，如果不想等待，请问下一个问题......'),
              ]
            : [TypewriterAnimatedText(message.text)],
        onTap: () {
          print("Tap Event");
        },
      ),
      color: _user.id != message.author.id ||
              message.type == types.MessageType.image
          ? const Color(0xfff5f5f7)
          : const Color(0xff6f61e8),
      margin: nextMessageInGroup
          ? const BubbleEdges.symmetric(horizontal: 6)
          : null,
      nip: nextMessageInGroup
          ? BubbleNip.no
          : _user.id != message.author.id
              ? BubbleNip.leftBottom
              : BubbleNip.rightBottom,
    );
